
#ifndef _SYSTEM_H
#define _SYSTEM_H

#define NULL ((void *)0)

/*
 * CPSR의 flag중 IRQ, FIQ를 사용허가/불가 한다.
 */
#define load_irq_enable()	\
({				\
unsigned long temp;		\
__asm__ __volatile__(		\
	"mrs %0, cpsr\n"	\
	"bic %0, %0, #0x80\n"	\
	"msr cpsr, %0\n"	\
	:"=r"(temp)		\
	:			\
	:"memory", "cc");	\
})

#define load_irq_disable()	\
({				\
unsigned long temp;		\
__asm__ __volatile__(		\
	"mrs %0, cpsr\n"	\
	"orr %0, %0, #0x80\n"	\
	"msr cpsr, %0\n"	\
	:"=r"(temp)		\
	:			\
	:"memory", "cc");	\
})
 

#define load_fiq_enable()	\
({				\
unsigned long temp;		\
__asm__ __volatile__(		\
	"mrs %0, cpsr\n"	\
	"bic %0, %0, #0x40\n"	\
	"msr cpsr, %0\n"	\
	:"=r"(temp)		\
	:			\
	:"memory", "cc");	\
})

#define load_fiq_disable()	\
({				\
unsigned long temp;		\
__asm__ __volatile__(		\
	"mrs %0, cpsr\n"	\
	"orr %0, %0, #0x40\n"	\
	"msr cpsr, %0\n"	\
	:"=r"(temp)		\
	:			\
	:"memory", "cc");	\
})

/*
 * 위의 매크로는 irq, fiq만을 지칭하지만
 * 아래의 매크로는 irq, fiq모두를 enable 혹은
 * disable 한다.
 */
#define ENTER_CRITICAL()	\
({				\
unsigned long temp;		\
__asm__ __volatile__(		\
	"mrs	%0, cpsr\n"	\
	"orr	%0, %0, #0xc0\n"\
	"msr	cpsr, %0\n"	\
	:"=r"(temp)		\
	:			\
	:"memory", "cc");	\
})
 

#define EXIT_CRITICAL()		\
({				\
unsigned long temp;		\
__asm__ __volatile__(		\
	"mrs	%0, cpsr\n"	\
	"bic	%0, %0, #0xc0\n"\
	"msr	cpsr, %0\n"	\
	:"=r"(temp)		\
	:			\
	:"memory", "cc");	\
})

void load_task(unsigned int task_number);

#endif
