
/*
 * software interrupt 부분으로 시스템 콜을 처리한다.
 * 
 * 만든사람: sukistyle freestyle, suh1978@gmail.com
 * 개정내용:
 * 참고사항: uart_gets를 제외하고는 standard getc, putc, puts와 동일하다고 보면됨
 */

#include "rtc.h"
#include "printk.h"
#include "string.h"

/*
 * 시스템 콜
 *
 * include/syscall.h에 시스템 콜들이 포장되어 있다.
 */
void version();			/* SWI 0 */
void rtc_get_date();		/* SWI 1 */
void rtc_set_date();		/* SWI 2 */

/*
 * Software Interrupt Handler
 */
void eventsSWIHandler(unsigned int syscallnum)
{
    switch(syscallnum) {

    case 0:			/* swi 0 */
	version();
	break;
	
    case 1:			/* swi 1 */
	rtc_get_date();
	break;

    case 2:			/* swi 2 */
	rtc_set_date();
	break;

    default:
	printk("SWI bad number: %d\n", syscallnum);
	
    }
}

/*
 * SWI 0: BlueOS into, kernel version print
 */
void version()
{
    printk("BlueOS info\n");
    printk(" - Kernel version: %s\n", VERSION);
    printk(" - Last Compile date: %s\n", __DATE__);
    printk("\nBlueOS developer: Suh Youn Suk\n");
    printk(" - E-Mail: suh1978@gmail.com\n");
    printk(" - BlueOS Homepage: blueOS.kldp.net\n");
}

/*
 * SWI 1: RTC를 이용한 현재 실제 일자/시간을 출력한다.
 */
void rtc_get_date()
{
    rtc_init();
    rtc_get();
    rtc_close();
}

/*
 * SWI 2: RTC를 이용한 현재 실제 일자/시간을 변경한다.
 */
void rtc_set_date()
{
    rtc_init();
    rtc_set();
    rtc_close();
}

