
/*
 * UART0 디바이스 드라이버
 * 
 * 만든사람: sukistyle freestyle, suh1978@gmail.com
 * 개정내용:
 * 참고사항: uart_gets를 제외하고는 standard getc, putc, puts와 동일하다고 보면됨
 */

#include "uart.h"

void uart_init()
{
    /*
     * UART0를 사용할것임.
     * nCTS0, nRTS0, TXD0, RXD0를 사용하겠다고 선언함.
     */
    rGPHCON &= ~0xff;
    rGPHCON |= 0xaa;
    rGPHUP |= 0xf;

    /*
     * ULCON0 = Uart Line Control Register
     * 내 미니컴의 환경설정 부분과 동일하게 설정함
     */
    rULCON0 = 0x03;

    /*
     * UCON0 = Uart Control Register
     * Transmit/Receive Mode set Interrupt request or polling mode
     * Uart Baud Rate use PCLK
     */
    rUCON0 = 0x245;

    /*
     * UFCON = Uart FIFO Control Register
     * UART0를 FIFO모드로 사용
     */
    rUFCON0 = 0x07;

    /*
     * UMCON = Uart Modem Control Register
     */
    rUMCON0 = 0x0;
    
    /*
     * UBRDIV0 = Uart Baud Rate Division Register
     * 
     * UBRDIV0 = (int)(PCLK/bps * 16)-1 or
     *           (int)(UCLK/bps * 16)-1
     */
    rUBRDIV0 = (int)(PCLK/(115200*16))-1;
}

char uart_getc()
{
    while (!(rUTRSTAT0 & 0x01))
	;

    return rURXH0 & 0x0ff;
}

/*
 * 실제 gets의 경우 버퍼의 크기를 넘어서는 버그가 있는데
 * 난 버퍼의 크기를 넘어서려고 하면 입력 받는것을 중지하고
 * 버퍼의 마지막에 '\0'을 무조건 넣어주었다
 * 이 함수는 에코된다
 */
char *uart_gets(char *buffer, int size)
{
    int i;

    for (i=0; i<size; i++) {
	/*
	 * 버퍼보다 큰 문자열을 넣으려고 하면
	 * 무조건 버퍼의 마지막에 0을 저장해서 리턴한다.
	 */
	if (i == size) {
	    buffer[i-1] = '\0';
	    return buffer;
	}
	
	buffer[i] = uart_getc();
	if (buffer[i] == 13) {	/* 13 = CR = '\r', 10 = LF = '\n' */
	    buffer[i] = '\0';
	    uart_putc('\n');
	    
	    return buffer;
	}
	uart_putc(buffer[i]);
    }

    return '\0';
}

void uart_putc(const char c)
{
    /*
     * transmit buffer register가 비어있을때까지 기다린다
     */
    while(!(rUTRSTAT0&0x2))
	;

    /*
     * 송신데이터 (8-비트)
     */
    rUTXH0 = c;
    
    if (c == '\n') {
	while(!(rUTRSTAT0&0x02))
	    ;
	rUTXH0 = '\r';
    }
}

void uart_puts(const char *s)
{
    while (*s)
	uart_putc(*s++);
}

